# mongodb-ui-components

### Prerequisites

You must have [node.js](https://nodejs.org/en/) ^5.0.0 installed. Were using 7.10 and would recommend installing the latest.

## Getting started

1. Run `npm install`
2. To run development mode: `npm run dev`
3. To build standalone version: `npm run build`
4. To install to a commonjs environment: `npm i @mongodb-js/mongodb-ui-components`

## Coding Style / Guidelines

[![js-standard-style](https://img.shields.io/badge/code%20style-standard-brightgreen.svg)](http://standardjs.com/)

This repo follows [standardjs](http://standardjs.com) guidelines.

## Development and Environments
Setting up a dev environment involves simply running `npm run dev` and accessing `localhost:2000` in your favorite browser, such as netscape navigator. Watchers are set up for edits made to JS and CSS.

## Components

### Code Widget

The code widget accepts the following props:

```js
const CodeWidget = require('@mongodb-js/mongodb-ui-components/build/code-widget')

<CodeWidget languages={[]} descriptions={{}} snippets={{}} />
```

`languages` is an optional array of strings. If a language isn't highlighting correctly, it should be added to this array which is used for fetching additional language scripts for highlight.js, which this widget uses under the hood. For example:

```js
<CodeWidget languages={['swift', 'malbolge', 'shakespeare', 'LOLCODE']} />
```

The structure of the description and snippets objects is as follows:

```js
// The description object houses the titles and text for the left 
// side of the widget, in which code snippets for any language 
// are explained in a general manner.
const descriptions = [
  {
    title: '',
    content: ''
  },
  // ...
]

const snippets = [
  language: 'Python',
  // Stages is a collection of strings representing code 
  // for each step.
  stages: [
    ``,
    // ...
  ],
  // sourceCode is a string containing the entire source for all stages
  // as well as any boilerplate code involved
  sourceCode: ``,
  // sourceUrl is a string pointing to a file of the original source code
  sourceUrl: 'https://website.com/source-code.cpp'
]
```

*It's important to note* that the minimap parser separates highlighted sections based on a sequence of numbered comments (e.g. # 1. Insert a document in python or // 3. Create a query). Make sure each step begins with a commented number in the stages array as well as the complete source.

You can find an example of how these properties are filled below.

```js
const descriptions = [
  {
    title: 'Connect',
    content: 'Here we are connecting to a locally hosted MongoDB database called <span class="monospace">test</span> with a collection named <span class="monospace">restaurants</span>.'
  }, {
    title: 'Insert a document',
    content: '5 example documents are being inserted into the <span class="monospace">restaurants</span> collection. Each document represents a restuarant with a name, star rating, and categories (stored as an array).'
  }, {
    title: 'Create a query',
    content: 'In this example, we run a simple query to get all of the documents in the <span class="monospace">restaurants</span> collection and store them as an array.'
  }, {
    title: 'Build an index',
    content: 'Indexes in MongoDB are similar to indexes in other database systems. MongoDB supports indexes on any field or sub-field of a document in a collection. Here, we are building an index on the name field with sort order ascending.'
  }, {
    title: 'Aggregate',
    content: 'Using MongoDB’s aggregation pipeline, you can filter and analyze data based on a given set of criteria. In this example, we pull all the documents in the restaurants collection that have a category of Bakery using the <span class="monospace">$match</span> operator and then group them by their star rating using the <span class="monospace">$group</span> operator. Using the accumulator operator, <span class="monospace">$sum</span>, we can see how many bakeries in our collection have each star rating.'
  }
]
```

```js
const snippets = [
  {
    language: 'Python',
    stages: [
`
# 1. Connect to MongoDB instance running on localhost
client = pymongo.MongoClient()

# Access the 'restaurants' collection in the 'test' database
collection = client.test.restaurants
`,
  `
# 2. Insert 
new_documents = [
  {
    "name": "Sun Bakery Trattoria",
    "stars": 4,
    "categories": ["Pizza","Pasta","Italian","Coffee","Sandwiches"]
  }, {
    "name": "Blue Bagels Grill",
    "stars": 3,
    "categories": ["Bagels","Cookies","Sandwiches"]
  }, {
    "name": "Hot Bakery Cafe",
    "stars": 4,
    "categories": ["Bakery","Cafe","Coffee","Dessert"]
  }, {
    "name": "XYZ Coffee Bar",
    "stars": 5,
    "categories": ["Coffee","Cafe","Bakery","Chocolates"]
  }, {
    "name": "456 Cookies Shop",
    "stars": 4,
    "categories": ["Bakery","Cookies","Cake","Coffee"]
  }
]

collection.insert_many(new_documents)
`,
  `
# 3. Query 
for restaurant in collection.find():
  pprint.pprint(restaurant)
`,
  `
# 4. Create Index 
collection.create_index([('name', pymongo.ASCENDING)])
`,
  `
# 5. Perform aggregation
pipeline = [
  {"$match": {"categories": "Bakery"}},
  {"$group": {"_id": "$stars", "count": {"$sum": 1}}}
]

pprint.pprint(list(collection.aggregate(pipeline)))
`
    ],
    documentation: [
      'https://document-link-for-step-1.com',
      'https://document-link-for-step-2.com',
      'https://document-link-for-step-3.com',
      'https://document-link-for-step-4.com',
      'https://document-link-for-step-3.com'
    ],
    sourceCode: ``
    sourceUrl: 'https://website.com/source-code.py'
  }, {
    language: 'C++',
    stages: cPlusPlusStages,
    documentation: [],
    sourceCode: ``
    sourceUrl: 'https://website.com/source-code.cpp'
  }, {
    language: 'Node.JS',
    stages: nodejsStages,
    documentation: [],
    sourceCode: ``,
    sourceUrl: 'https://website.com/source-code.js'
  }
]
```

### MongoDB Nav

This nav displays different content depending on the environment given to its props.

```js
const React = require('react')
const { render } = require('react-dom')
const Nav = require('@mongodb-js/mongodb-ui-components/build/nav')

render(<Nav
  // If the nav should have a dark background on load
  isOpaque={false} 
  
  // Where search results should redirect
  searchUrl='https://www.google.com/#q='
  
  // If the nave should have search features
  includeSearch={true}
  
  // The querystring appended to the download button link, for data collection
  downloadJmpValue='docs'
  
  // The location of the mongodb logo asset
  logoUrl='https://webassets.mongodb.com/_com_assets/global/mongodb-logo-white.png'
  
  // Where the logo will redirect when clicked
  logoLink='https://docs.mongodb.com'
  
  // The text that accompanies the MongoDB logo
  tagline='Documentation'
  
  // An array of objects representing the list of items on the top nav bar
  topSections={[
    {
      text: 'Docs',
      href: 'https://docs.mongodb.com'
    }, {
      text: 'Open Source',
      href: '/community'
    }, {
      text: 'University',
      href: 'https://university.mongodb.com'
    }
  ]}

  // The middle section of the nav
  // Setting "isLink" to true will remove any dropdown behavior,
  // see how the subsections correspond
  sections={[
    {
      text: 'Server',
      href: 'https://docs.mongodb.com/manual',
      isLink: true
    }, {
      text: 'Drivers',
      href: 'https://docs.mongodb.com/ecosystem/drivers/',
      isLink: true
    }, {
      text: 'Cloud Services'
    }, {
      text: 'Compass',
      href: 'https://docs.mongodb.com/compass',
      isLink: true
    }, {
      text: 'BI Connector',
      href: 'https://docs.mongodb.com/bi-connector',
      isLink: true
    }
  ]}

  // The dropdown lists based on the middle nav sections
  subsections={[
    null,
    null,
    {
      cta: {
        header: 'Try It Now',
        links: [
          {
            text: 'MongoDB 3.4',
            href: 'https://www.mongodb.com/mongodb-3.4?jmp=docs'
          }, {
            text: 'Download Center',
            href: 'https://www.mongodb.com/download-center?jmp=docs'
          }
        ]
      },
      linkLists: [
        [
          {
            href: 'https://docs.cloud.mongodb.com/',
            text: 'Cloud Manager',
            bold: true
          },
          {
            href: 'https://docs.opsmanager.mongodb.com/current',
            text: 'Ops Manager',
            bold: true
          }
        ]
      ]
    },
    null,
    null
  ]} />, someElement)
```

The content it decides to display is set in the `/components/nav/jsx/environments/` directory.

Each environment exports an object containing two properties: sections and subsections.

Sections is an array of objects; subsections is an array of [jsx](https://facebook.github.io/react/docs/jsx-in-depth.html) trees.

Sections are displayed right underneath the top white nav. Subsections are the in-detailed subnavs that are toggled by selecting sections.

The order of each section in the array must correspond with each subsection. If a specific section is merely a link and isn't meant to toggle a subsection, just set its corresponding subsection to `null` (not `undefined`!).

An example for structuring sections and subsections can be found in the testing html file, `/public/index.html`

#### How should I incorporate this into my website?

#### The Best Way

The "best way" makes a few key assumptions:

1. You are using React in your project
and / or
2. You are using a module bundler like webpack

***Disclaimer***: this is the more technically challenging method, but you get the largest payoff in terms of performance and development agility :)

The absolutely hands-down best way to integrate this nav is via an isomorphic javascript architecture. In this case, this means rendering the Nav Component server side using react's `ReactDOMServer.renderToString()` method, and then hooking up the component client side by calling `ReactDOM.render()` on the Nav's parent with its initial server-side props.

If you don't use an isomorphic rendering architecture, rendering on the client will work just fine, but you'll need to find a way to handle search engine optimization.

We understand that this may not be possible for every team's architecture, which is why we have the "good way".

#### The Good Way

The "good way" makes a few key assumptions:

1. You aren't using React in your project.
2. You aren't using a module bundler like webpack or browserify.

First, simply build the nav by typing `npm run build` into the terminal of the Commodore 64 MongoDB provided you when you were hired.

From this point on, the public folder is your friend. It contains an `index.css` and `index.js` script which will be used for including your very own shiny new MongoDB Navigation Bar&copy;. `index.html` gives a good example for how to assimilate said nav, and we'll quickly walk you through it: specifically what's going on with our `<script>`.

```html
<script>
// Loading the script, not much is special here
var script = document.createElement('script')
script.async = true
script.onload = onload
script.src = 'index.js'

document.head.insertBefore(script, document.head.children[0])

// Once the script loads, a global object will become 
// available with a render method. 
function onload () {
  /**
   * @param {string} parentNode - the node in which to render the nav
   * @param {object} props - the props passed to the nav component (see above)
   * @param {function} next - callback for when the nav has finished rendering
   */
  window.MongoNav.render('#nav-root', {
    isOpaque: true,
    searchUrl: 'https://www.google.com/#q=',
    includeSearch: true,
    downloadJmpValue: 'docs',
    logoUrl: 'https://webassets.mongodb.com/_com_assets/global/mongodb-logo-white.png',
    logoLink: 'https://docs.mongodb.com',
    tagline: 'Documentation',
    topSections: [ /* ... */ ],
    sections: [ /* ... */ ],
    subsections: [ /* ... */ ]
  }, function () {
    // Everything's rendered!
  })
}
</script>
```


#### Search Engine Optimization
We need this nav to be available to search engines and thus it needs to be available in the HTML before client-side JS is run. If you're using the isomorphic method mentioned above, you don't need to worry about anything, this is already handled for you and you probably already know that!

There isn't a pretty way to do this if you're not using the "best way". You'll have to generate the nav in a test environment on your site, peek into dev tools, copy all html within your `<div id='#react-nav-root'></div>` container, and then paste it into your html that's sent from the server as a placeholder until the client-side JS runs. We're working on writing a simple generator that'll make prettier html for you on command, so stay tuned!
